@@ -51,12 +51,47 @@ test_that ("categorical dists", {
     expect_true (all (dims [2, ] == nt))
 
     expect_message (
-        d2 <- dodgr_dists_categorical (graph, from = from, to = to, quiet = FALSE),
+        d2 <- dodgr_dists_categorical (
+            graph,
+            from = from,
+            to = to,
+            quiet = FALSE
+        ),
         "Calculating shortest paths ..."
     )
     expect_identical (d, d2)
 })
 
+test_that ("categorical dists results", {
+
+    net <- weight_streetnet (hampi, wt_profile = "foot")
+    v <- dodgr_vertices (net)
+    set.seed (1L)
+    from <- sample (v$id, 20)
+    to <- sample (v$id, 20)
+
+    net$edge_type <- net$highway
+    d0 <- dodgr_dists_categorical (net, from, to, proportions_only = FALSE, pairwise = FALSE)
+    # Sums of all "type" matrices should equal main "distances" matrix:
+    types <- which (names (d0) != "distances")
+    d_types <- lapply (types, function (i) colSums (d0 [[i]]))
+    d_types <- colSums (do.call (rbind, d_types))
+    d_total <- colSums (d0$distances)
+    expect_true (all (abs (d_total - d_types) < 1e-6))
+
+    d1 <- dodgr_dists_categorical (net, from, to, proportions_only = FALSE, pairwise = TRUE)
+    index_rows <- which (d1 [, 1] > 0)
+    index_cols <- which (colnames (d1) != "total")
+    d_total <- d1 [index_rows, 1L]
+    d_types <- rowSums (d1 [index_rows, index_cols])
+    expect_true (all (abs (d_total - d_types) < 1e-6))
+
+    p0 <- dodgr_dists_categorical (net, from, to, proportions_only = TRUE)
+    dp <- vapply (d0, function (i) sum (colSums (i)), numeric (1L))
+    p1 <- dp [-1] / dp [1]
+    expect_true (all (abs (p0 - p1) < 1e-6))
+})
+
 test_that ("categorical dists summary", {
 
     expect_silent (graph <- weight_streetnet (hampi))
@@ -125,3 +160,35 @@ test_that ("categorical threshold", {
         expect_true (all (d [, i] <= d [, 1]))
     }
 })
+
+test_that ("categorical pairwise", {
+
+    expect_silent (graph <- weight_streetnet (hampi))
+    graph <- graph [graph$component == 1, ]
+
+    v <- dodgr_vertices (graph)
+    set.seed (1L)
+    from <- sample (v$id, size = 100)
+    to <- sample (v$id, size = 100)
+
+    graph$edge_type <- graph$highway
+
+    expect_silent (
+        d <- dodgr_dists_categorical (graph, from, to, pairwise = TRUE)
+    )
+
+    expect_is (d, "matrix")
+    expect_equal (nrow (d), length (from))
+    expect_equal (ncol (d), length (unique (graph$edge_type)) + 1L)
+
+    # d [, 1] is distance; all other cols are edge-type-specific
+    for (i in 2:ncol (d)) {
+        expect_true (all (d [, i] <= d [, 1]))
+    }
+
+    # rownames are "<from>-<to>":
+    from_id <- gsub ("\\-.*$", "", rownames (d))
+    to_id <- gsub ("^.*\\-", "", rownames (d))
+    expect_identical (from_id, from)
+    expect_identical (to_id, to)
+})
